;HERO VARIABLES
HERO_IDLE=ENEMY_IDLE
HERO_MOVE_LEFT=ENEMY_LEFT
HERO_MOVE_RIGHT=ENEMY_RIGHT
HERO_MOVE_DOWN=ENEMY_DOWN
HERO_MOVE_UP=ENEMY_UP
HERO_FALL=ENEMY_FALL

HERO_DEADCLR=14

FACE_RIGHT=1
FACE_LEFT=2

DIR_NULL=0
DIR_UP=1
DIR_DOWN=2


 STRUCT OBJHERO
; Common fields between the hero and the enemies
xPos		db	0
yPos		db	0	; 209=disabled, 200=in limbo
moveCounter	db	32
state		db	0
mode		db	MODE_PATROL_X	; Autoplay only
faceDir		db	FACE_RIGHT
prevMode	db	MODE_PATROL_X	; Autoplay only
rndSeedCounter	db	24		; Autoplay only
turnCounter	db	0		; Autoplay only
sprAnimDelay	db	0
deathTimer	db	DEADENDTTL	; Autoplay only
; Specific fields for the hero
Unused2		db	0	; Unused on the hero
isHero		db	0	; 0=hero, nz=enemy
prevState	db	HERO_IDLE
animCounter	db	0
 ENDS


resetHero:
	ld	hl,baseEnemies
	ld	de,hero
	ld	bc,OBJHERO
	ldir
	ld	ix,hero
	ld	iy,baseEnemies
	call	initRndGen

	xor a : ld (hero.state),a : ld (hero.prevState),a : ld (blockTick),a
	ld (heroGold),a : ld (heroPickaxe),a : ld (hero.moveCounter),a
	ld hl,00000 : ld (hero.xPos),hl
	ld a,FACE_RIGHT : ld (hero.faceDir),a
	ret

doHero:    

	;process hero blocks
	call	processBlock

	call	heroControls		
	call	doHeroSounds

	ld	a,(hero.state)
	ld	(hero.prevState),a
	call	doHeroMove
	call	doSprAnimationTiming
	call	calcHeroSprite		


	;	Check hero against enemy coordinates
	ld	b,MAXENEMIES
	ld	iy,enemiesList
1:
	call	checkEnemy
	ld	de,OBJENEMY
	add	iy,de
	djnz	1b

	ret

checkEnemy:
	;if we blink - not kill
	ld a,(iy+OBJENEMY.moveCounter)
	cp 33
    ret nc
;X
	ld a,(hero.xPos)        ; player's x coordinate.
	sub (iy+OBJENEMY.xPos)          ; subtract alien x.
	add a,3             ; player is 3 high, so add 3 - 1 = 2.
	cp 6                ; combined heights are 3 + 2 = 5.
    ret nc              ; not within vertical range.	

	;Y
	ld a,(hero.yPos)        ; player's y coordinate.
    sub (iy+OBJENEMY.yPos)          ; subtract alien y.
    add a,4             ; player is 3 wide, so add 3 - 1 = 2.
    cp 6                ; combined widths are 3 + 4 = 7.
    ret nc              ; not within horizontal range.
    ;jp collis           ; we have a collision.

	;kill hero
	jp killHero

;checkFireButton:	  
;	ld	a,(joyStatus)
;	bit	4,a
;	ret	z
;	bit	5,a
;	ret	z
;	ret

doAction:
	;check if was fire button up
;	ld a,(wasFireUp)
;	or a
;	ret z
	
	ld a,(hero.state)
	cp HERO_IDLE
	ret nz

	;reset fireup state to fire pushed
;	xor a : ld (wasFireUp),a
	ld	a,#FF
	ld	(joyPressedGam),a

	;pause hero
	ld a,8 : ld (hero.moveCounter),a
	ld a,HERO_IDLE : ld (hero.state),a

	ld de,(hero.xPos)
	call getTileCoords
	inc d

	;not dig under ladder
	ld a,(hl): cp TILE_LADDER : jp z,1f

	ld bc,32 : add hl,bc : ld a,(hl)	
	;no free space check if block to remove
	cp TILE_SOLID+1
	jr nz,1f
	
	;check if we have pickaxe
	ld a,(heroPickaxe)
	or a
	ret z
	exx : call subPickaxe : exx

	ld (hl),0 : jp hideBlock

1:

;bridge checks
	ld	a,(gameState)
	cp	GS_BRIDGE	; A bridge is already being extended?
	ret	z		; Yes, quit

	ld	a,(hl)
	cp TILE_LEFT
	jr z,4f
	cp TILE_RIGHT
	ret nz
	ld a,FACE_RIGHT : ld (bridgeFace),a
	ex af,af :	ld a,TILE_ANTIRIGHT;disable arrow
	ld (hl),a;TILE_ANTILEFT;disable arrow
	ex af,af
	jr 5f
4:
	ld a,FACE_LEFT : ld (bridgeFace),a
	ex af,af : ld a,TILE_ANTILEFT;disable arrow
	ld (hl),a;TILE_ANTILEFT;disable arrow
	ex af,af
5:
	;add TILE_ANTILEFT-1
	
	ld a,8 : ld (hero.moveCounter),a : call playMostSound	
	ld de,(hero.xPos)
	call getTileCoords : inc d
	ld bc,32 : add hl,bc
	push de : push hl
	ex af,af :  call fillTile : ex af,af

	pop hl : pop de
	
	ld a,(bridgeFace)
	jp initBridge


heroControls:
	;controls and physic
	ld	a,(currentLevel)
	cp	LASTLEVEL+1
	ret	nc

	ld	a,(gameState)
	cp	GS_GAMEPLAY
;	ret	nz
	jr	z,10F
	cp	GS_BRIDGE
	ret	nz

10:	call	moveControls    


	ld a,(hero.state)
	cp HERO_IDLE
	ret nz

	ld de,(hero.xPos)
	call getTile
	ld bc,32 : add hl,bc
	ld a,(hl)
	cp TILE_SOLID+1
	ret c

2:	  
	ld	a,(joyPressedGam)
	and	#10
	ret	nz
	jp	doAction


//check falling down
checkFall:
	;check fi we on ladder
	ld de,(hero.xPos)	
	call getTileCoords
	ld	a,(hl)
	cp TILE_LADDER : ret z

	;check tile under hero
	ld bc,32 : add hl,bc
	ld	a,(hl)

	cp TILE_MINE
	jr nz,1f
	push af

	; Activate mine
	ld (hl),TILE_ACTIVEMINE
	inc d
	push de : call addMine : pop de	

	pop af
1:

	cp TILE_SOLID :  ret nc
	
	ld a,HERO_FALL : ld (hero.state),a	: ld a,4 : ld (hero.moveCounter),a	
	ret

collectGold:	
	ld (hl),0
	ld de,(hero.xPos)
    xor a : call fillTileXY
	;call getTileCoordsDown : ld a,0
	;ld (topTile1+1),a : ld (topTile1Coords+1),de
	call playCollectSound2
	call addGold
	xor a;reset tile
	ret

collectPickaxe:
	ld (hl),0
	ld de,(hero.xPos)
    xor a : call fillTileXY
	;call getTileCoordsDown : ld a,0
	;ld (topTile1+1),a : ld (topTile1Coords+1),de
	call playCollectSound
	call addPickaxe
	xor a;reset tile
	ret

heroWin:	
;	ld a,(gameState) : cp GS_GAMEPLAY : ret nz
	ld a,GS_STAGECLEAR : ld (gameState),a	
	jp	muteMusic

teleportHero:	
	ld a,12 : ld (hero.yPos),a
	jp	teleportSound

killHero:	
	ld a,GS_DEAD : ld (gameState),a
	ld	a,HERO_DEADCLR
	call	setHeroColor
	jp	muteMusic	

endMove:
	;check dead on fire and pike
	ld	de,(hero.xPos)
	ld	a,e
	and	#F8		; Don't let errors accumulate
	ld	(hero.xPos),a
	ld	e,a
	ld	a,d
	sub	4
	and	#F8		; Don't let errors accumulate
	add	4
	ld	(hero.yPos),a
	ld	d,a

	call getTile
	cp TILE_TELEPORT
	jp z,teleportHero
	cp TILE_FIRE
	jp z,killHero
	cp TILE_PIKE
	jp z,killHero

	;set idle state
	ld a,HERO_IDLE
	ld (hero.state),a
	;check end of level
	ld a,(heroGold)	
	ld bc,(totalGold)
	cp c
	jp z,heroWin
	;check for falling
	jp checkFall	

doHeroMove:
	ld	a,(hero.moveCounter)
	or	a
	jr	z,endMove
	dec	a
	ld	(hero.moveCounter),a	
	cp	2
	jr	nc,.moveSel
	cp	1
	jr	z,.moveSel
1:	;on step 0 and 2 check collect
;	push	af
	ld	de,(hero.xPos)	
	call	getTile	
	CP	TILE_GOLD
	call	z,collectGold	
	CP	TILE_PICKAXE
	call	z,collectPickaxe	
;	pop	af
;	or	a
;	jr	z,endMove

.moveSel:
	ld a,(hero.state)
	cp HERO_MOVE_RIGHT
	jp z,moveHeroRight
	cp HERO_MOVE_LEFT
	jp z,moveHeroLeft
	cp HERO_MOVE_UP
	jp z,moveHeroUp
	cp HERO_MOVE_DOWN
	jp z,moveHeroDown
	cp HERO_FALL
	jp z,fallHero
	ret

moveHeroUp:
	ld a,(hero.yPos)
	dec a
	ld (hero.yPos),a
	ret
moveHeroDown:
	ld a,(hero.yPos)
	inc a
	ld (hero.yPos),a
	ret
fallHero:
	ld	a,#FF
	ld	(joyPressedGam),a
	ld a,(hero.yPos)
	inc a:inc a
	ld (hero.yPos),a
	cp	192-6
	ret	c
	ld a,12 : ld (hero.yPos),a
	ret

moveHeroRight:
	;set face direction
	ld a,FACE_RIGHT : ld (hero.faceDir),a

	ld a,(hero.xPos)
	inc a
	ld (hero.xPos),a
	ret
moveHeroLeft:
	;set face direction
	ld a,FACE_LEFT : ld (hero.faceDir),a

	ld a,(hero.xPos)
	dec a
	ld (hero.xPos),a
	ret




moveControls:
	ld	a,(hero.state)
	cp	HERO_FALL
	ret	z

	ld	a,(gameSubState)
	cp	GSS_GAMEPLAY_DEMOPLAY
	jr	z,.autoplay

	ld	a,(hero.moveCounter)
	or	a
	ret	nz	  	

	;reset last dir
;	xor	a: ld	(lastDir),a

	ld	a,(joyStatus)
	; Decode left/right with higher priority, so the hero doesn't stop
	; when diagonals are pressed.
	bit	2,a
	jp	z,tryMoveLeft
	bit	3,a
	jp	z,tryMoveRight
	bit	0,a
	jp	z,tryMoveUp
	bit	1,a
	jp	z,tryMoveDown

	ld	a,HERO_IDLE
	ld	(hero.state),a

	ret

.autoplay:
	; Check for a key to end the demoplay before we simulate buttons
	ld	a,(joyStatus)
	cpl
	or	a			; Any button pressed?
	jr	z,5F			; No, skip
	ld	hl,0
	ld	(demoTimer),hl

5:	; Check if the hero is idle and has no more movements left
	ld	a,(hero.moveCounter)
	or	a
	ret	nz
	ld	a,(hero.state)
	cp	HERO_IDLE
	ret	nz

	ld	a,(hero.prevState)
	ld	(hero.state),a

	ld	ix,hero
	ld	e,(ix+OBJHERO.xPos)
	ld	d,(ix+OBJHERO.yPos)
	call	doLogic.part2
	ld	(ix+OBJENEMY.moveCounter),a

	; Check if the AI was killed 
	ld	a,(hero.deathTimer)
	cp	128
	jp	z,killHero 
	;
	; Complement the functions missing on the enemy AI:
	; dig and jump from bridges
	ld	a,(hero.state)
	cp	HERO_IDLE
	ret	nz

	; Check the tiles below the hero
	ld	e,(ix+OBJHERO.xPos)
	ld	a,(hero.yPos)
	add	8
	ld	d,a
	call	getTile
	cp	TILE_BRIDGE
	jr	z,50F
	cp	TILE_SOLID+1
	ret	nz

40:	; Try to dig a hole
	ld	a,(heroPickaxe)
	or	a
	ret	z
	ld	a,(hero.turnCounter)
	cp	DEADENDTTL/2		; Probably stuck?
	jr	c,44F			; At least try to dig out
	call	getEnemyRandom
	or	a
	ret	nz	
44:	ld	hl,joyStatus		; Simulate a button press
	res	4,(hl)
	inc	hl
	res	4,(hl)
	inc	hl
	res	4,(hl)
	ret

50:	; Try to jump from bridges
	ld	a,(hero.turnCounter)
	cp	DEADENDTTL/3		; Bumping needlessly?
	jr	nc,54F			; Get out 
	call	getEnemyRandom
	or	a
	ret	nz	
54:	ld	hl,joyStatus		; Simulate a button press
	ld	a,#FD
	ld	(hl),a
	inc	hl
	ld	(hl),a
	inc	hl
	ld	(hl),a
	xor	a
	ld	(hero.turnCounter),a
	jr	tryMoveDown



tryMoveRight:
	;if we don't stand - ret
   	ld a,(hero.state)
	cp HERO_IDLE
	ret nz
	;change hero sprite
	;ld hl,heroRight : ld (heroSprite+1),hl

	;check collisions
	ld de,(hero.xPos)
	ld a,e : add 4 : ld e,a
	call getTile	

	;if not solid blocks then move
	cp TILE_SOLID
	ret nc

	;change hero state
	ld a,HERO_MOVE_RIGHT : ld (hero.state),a	: ld a,8 : ld (hero.moveCounter),a		
    ret

tryMoveLeft:
	;if we don't stand - ret
   	ld a,(hero.state)
	cp HERO_IDLE
	ret nz
	;change hero sprite
	;ld hl,heroLeft : ld (heroSprite+1),hl

	;check collisions
	ld de,(hero.xPos)
	ld a,e : sub 8+4 : ld e,a
	call getTile	
	
	;if not solid blocks then move
	cp TILE_SOLID
	ret nc
	
	;change hero state
	ld a,HERO_MOVE_LEFT : ld (hero.state),a	: ld a,8 : ld (hero.moveCounter),a		

    ret

tryMoveUp:
	;write last dir
;	ld a,DIR_UP : ld (lastDir),a

	;if we don't stand - ret
   	ld a,(hero.state)
	cp HERO_IDLE
	ret nz

	;check if there is a ladder	
	ld de,(hero.xPos)
	;ld a,d : sub 8 : ld e,d
	call getTile		

	cp TILE_LADDER : ret nz

	;is there solid above head&
	ld bc,-32:add hl,bc ;up not solid
	ld a,(hl)
	
	;if not solid blocks then move
	cp TILE_SOLID
	ret nc

	cp TILE_LADDER
	ret nz

	;change hero sprite
	;ld hl,heroLadder : ld (heroSprite+1),hl

	;change hero state
	ld a,HERO_MOVE_UP : ld (hero.state),a	: ld a,8 : ld (hero.moveCounter),a	

    ret

tryMoveDown:
    	;if we don't stand - ret
   	ld a,(hero.state)
	cp HERO_IDLE
	ret nz

	;write last dir
;	ld a,DIR_DOWN : ld (lastDir),a

	;check if there is a ladder	down
	ld de,(hero.xPos)
	ld a,d : add 8 : ld d,a
	call getTile	

	or a: jr z,1f

	;check bridge, jump from bridge	
	ld	b,a			; b=tile under hero
	ld	a,(joyPressedGam)
	and	#02			; Was down pressed in this cycle?
	ld	a,b
	jr	nz,10F			; Won't chain two bridge jumps
				; TODO: Force the down key to be released 
	cp TILE_BRIDGE : jr z,2f
	cp TILE_BRIDGE1 : jr z,2f
	cp TILE_BRIDGE2 : jr z,2f

	 ;jump from ladder
10:	cp TILE_LADDER : ret nz
1:
	;change hero sprite
	;ld hl,heroLadder : ld (heroSprite+1),hl
	;change hero state
	ld a,HERO_MOVE_DOWN : ld (hero.state),a	: ld a,8 : ld (hero.moveCounter),a	
	ret

2:	;fall from bridge
	ld	a,#FF
	ld	(joyPressedGam),a
	ld a,HERO_FALL : ld (hero.state),a	: ld a,8 : ld (hero.moveCounter),a	
    ret

20:


;Process basic animations deltas
doSprAnimationTiming:
;DELAY
	ld a,(hero.sprAnimDelay)
	inc a
	and 7
	ld (hero.sprAnimDelay),a
	or a
	ret nz
        ;tick for basic animations
    	ld a,(hero.animCounter)
	inc a : and 1
	ld (hero.animCounter),a
	ret


;=============== BLOCK ==================

//Process show|hide block animation (brick disappear)
processBlock:
	ld a,(blockTick)
	or a
	ret z
	dec a
	ld (blockTick),a
	
	;delay
	ld a,(blockDelay)
	inc a: and 1
	ld (blockDelay),a
	or a
	ret z

	ld a,(blockFrame)
	dec	a
	jp	m,30F
	cp	14
	jr	nc,50F
30:	xor	a		; Empty space
50:	ld (blockFrame),a
    ld de,(blockCoords)
	;patch up draw
	jp	fillTile

hideBlock:
	;xor a: call fillTile
	ld a,16
	ld (blockFrame),a
	ld a,8 : ld (blockTick),a
	ld (blockCoords),de
	jp	playHideBlockSound

