
; Note: The code is formatted with TABSTOP=8, the standard for assembly

START_LEVEL=1		; start level
START_LIVES=5		; Number of heroLives
MAXENEMIES=31		; Max number of enemies (from 3 to 31)
MAXMINES=8		; Max number of simultaneouly active mines

 DEFINE TGTFRMSKP 0	; Desired number of frameskips
 DEFINE MAXFRM 05h	; Set the maximum number of lost frames that
			; will be remembered
 DEFINE MENUWAIT 2*44*50	; Time to wait on the menu (2 music loops)
 DEFINE DEMOWAIT 50*60	; 1 minute
; DEFINE SHOWENEMYMODE	; Change the color of the enemy according to its mode
 DEFINE SHOWCLRENEMY	; Use one color for each enemy, for debugging
; DEFINE PROFILER	; Use the screen border for CPU time profiling (MSX>=2 only)

 DEFINE JOYLIB.KBDBTN2GRAPH

fxData		equ	musicBuffer 		; Transiction FX table
MUSBUFFERSIZE	equ	#1800-(map.end-map)	; Music buffer size
outroText	equ 	musicBuffer+MUSBUFFERSIZE/2
FCBPNT		equ	musicBuffer		; FCB area
fileBuf		equ	FCBPNT+DOSFCB
EZFILELIB.WHICHBDOS	equ	BDOSBAS

;(+) ESC - restart
;(+) remove keyboard clicks
;(+) NTSC \ PAL optimization
;(+) joystick
;(+) change the color of tiles
;(+) menu
;(+) final
;(+) gui
;(+) cheat buttons and restart

;gameplay states
GS_GAMEPLAY=0
GS_BRIDGE=1
GS_FADEOUT=2
GS_STAGECLEAR=3
GS_DEAD=4
GS_GAMEOVER=5
GS_STARTLEVEL=6
GS_SHOWLEVEL=7
GS_RESTARTLEVEL=8
GS_SHOWLEVELQUICK=9

;gameplay sub states
GSS_GAMEPLAY_INTERACT=0
GSS_GAMEPLAY_DEMOPLAY=1
GSS_GAMEPLAY_USRLEVEL=2


    device ZXSPECTRUM48;it maps pages just 0,1,2,3

    ;Cart mapped in $4000
    org $4000
Start:
    include "include\cartHeader.inc" ;include a header that point to Start

boot:
;	ld	a,(HPHYD)
;	cp	0C9h		; Is there a disk interface present?
;	jr	nz,init		; Yes, do a warmboot

coldBoot:
	di
	call	BASEMSXLIB.getfrm1slot
	ld	h,a
	ld	l,#F7		; RST30h opcode
	ld	(HSTKE),hl	; set rst30h and slot on hook
	ld	hl,init
	ld	(HSTKE+2),hl	; set cartboot on hook
	ret			; return and wait for the boot to finish

init:	ld	sp,#E000
	call	initGame
intro:	call	muteMusic
	call	showIntroPicture
	ld	hl,menuMusic
	call	initMusic
	call	doMenu
	jr	c,intro
	call	initGameplay    
	
mainLoop:
	call	dynVsync
	call	gameLoop

 IFDEF PROFILER
	ld	bc,#0107	; Black = end
	call	WRTVDP
 ENDIF ; PROFILER
	jr	mainLoop

popintro:
	pop	af	; Discard a previous return point
	jr	intro

gameLoop:
 IFDEF PROFILER
	ld	bc,#0207	; Game Loop = Green
	call	WRTVDP
 ENDIF ; PROFILER
 
    ;animate fire and gold
	call animateFire
	call animateGold

	ld	a,(gameState)
	cp	GS_GAMEPLAY
	jp	z,doGameplay
	cp	GS_BRIDGE
	jp	z,doBridge

	cp	GS_SHOWLEVEL
	jp	z,doShowLevel

	cp	GS_SHOWLEVELQUICK
	jp	z,showLevelQuick

	cp	GS_STARTLEVEL
	jp	z,doStartLevel

	cp	GS_RESTARTLEVEL
	jp	z,doRestartLevel

	cp	GS_STAGECLEAR
	jr	z,doStageClear

	cp	GS_DEAD
	jp	z,doStageDead  

	cp	GS_GAMEOVER
	jp	z,doGameOver

	ret

doStageClear:
	; Reuse deadCount to wait for some time after the stage is cleared
	ld a,(deadCount) : dec a : ld (deadCount),a
	or a
	jr z,1f	

	cp	73
	ret	nz
	ld	a,13
	jp	AFXPLAY3
1:
	ld	a,(gameSubState)
	cp	GSS_GAMEPLAY_DEMOPLAY
	jp	z,popintro
	cp	GSS_GAMEPLAY_USRLEVEL
	jp	z,popintro

	ld a,75 : ld (deadCount),a
    ld a,(currentLevel): inc a : ld (currentLevel),a	

	ld	a,GS_STARTLEVEL
	ld	(gameState),a
	ld	a,GSS_GAMEPLAY_INTERACT
	ld	(gameSubState),a

	; Check if the previous stage was skipped
	ld	hl,gameFlags
	bit	gameFlags.skipd,(hl)
	res	gameFlags.skipd,(hl)
	ld	a,START_LIVES
	jr	nz,5F		; Yes, skip
	;  Extra lives every 4 stages
	ld	a,(currentLevel)
	dec	a
	and	3
	ret	nz
	ld	a,16
	call	AFXPLAY3
	ld	a,(heroLives)
	inc	a
5:	ld	(heroLives),a
	jp	drawLive


doRestartLevel:
	ld	a,GS_SHOWLEVELQUICK
	jr	doStartLevel.setstate

doStartLevel:
	ld	a,GS_SHOWLEVEL
.setstate:
	ld	(gameState),a
	call	muteMusic    
	;init level
	call	clearAllSprites
	ld	a,(gameSubState)
	cp	GSS_GAMEPLAY_USRLEVEL
	jr	z,doStartUsrLevel

	call	loadLevelROM
	ld	a,(gameState)
	cp	GS_SHOWLEVEL
	call	z,drawLevelTitle
	jp	initLevel   

doStartUsrLevel:
	call	loadLevelFile
	jp	c,popintro
	ld	a,(gameState)
	cp	GS_SHOWLEVEL
	call	z,drawLevelTitle
	jp	initLevel


doStageDead:
	ld a,(deadCount) : dec a : ld (deadCount),a
	or a
	jr z,1f	

	; Check for the demoplay on the 1st cycle
	cp	73
	ret	nz
	ld	a,14
	call	AFXPLAY3
	ld	a,(gameSubState)
	cp	GSS_GAMEPLAY_DEMOPLAY
	jp	z,popintro
	ret

1:
	ld a,75 : ld	(deadCount),a
	ld	a,GS_RESTARTLEVEL
	ld	(gameState),a
	ld	a,(heroLives) : dec	a
	ld	(heroLives),a
	push	af
	call	drawLive
	pop	af
	ret	nz
	;
	ld	a,GS_GAMEOVER
	ld	(gameState),a
	ld	hl,0
	ld	(timer),hl
	ld	a,208
	ld	hl,(GRPATR)
	call	WRTVRM		; Hide all sprites 
	ld	hl,musgameover
	call	initMusic
  	ld	de,10*32+11
	call	.fillBorder
	ld	bc,gameOverText
	call	drawString
    	ld	de,12*32+11
;	jp	.fillBorder

.fillBorder:
	ld	hl,(GRPNAM)
	add	hl,de
	ld	bc,gameOverText.end-gameOverText-3
	xor	a
	jp	FILVRM

;-----------------------------
initOutro:
	ld	hl,outro4
	ld	de,outroText
	call	DEC40

doDelayedOutro:
	ld	a,(outroCount)
	cp	1			; At least one time?
	jr	c,10F			; No, quit
	ld	a,(joyStatus)
	and	#10
	jp	z,popintro
	ld	a,(outroCount)
	cp	3			; Repeated 3 times?
	jp	nc,popintro		; Ok, that's enough

10:	ld a,(outroDelay) : inc a : ld (outroDelay),a
	or a : ret nz
doOutro:	
	ld bc,(outroString)
	push bc
	call drawString
	pop hl : ld bc,29: add hl,bc
	ld a,(hl)
	cp 254
	jr nz,10f
	ld hl,outroText
	ld	a,(outroCount)
	inc	a
	ld	(outroCount),a
10:
	ld (outroString),hl
	ret


doGameplay:
    ;copy 32 sprites attributes to VRAM
    call doSprites  
    ;processHero
    call doHero

    ;do enemies
    call doEnemies  

    ;show outro
    ld a,(currentLevel)
	cp	LASTLEVEL+1
	jp	nc,initOutro

    ld a,(globalMineDelay)
    inc a : and 1 : ld (globalMineDelay),a
    or a
    jr nz,1f
    ;===================================
    call doKabum
    call doMines
    ;===================================
1:

	ld	a,(gameSubState)
	cp	GSS_GAMEPLAY_DEMOPLAY
	jr	nz,9F

	; Demoplay Mode
	ld	hl,(demoTimer)
	dec	hl
	bit	7,h		; Time to end the demoplay?
	jr	z,8F		; no, skip
	jp	popintro
8:	ld	(demoTimer),hl
	ret

	; ESC=restart level
9:	ld	a,(fnBtnPressed)
    	bit	5,a		; ESC pressed?
	jp	z,.restartLevel

	; Pause
	bit	7,a		; STOP pressed?
	jr	nz,80F
	call	pauseSound
10:	call	waitVBL		; Wait for a keyboard update
	ld	a,(fnBtnPressed)
	and	#80		; STOP pressed?
	jr	nz,10B
	ld	a,(joyStatus)
	and	#10		; button-1 pressed?
	jr	z,.restartLevel	; Yes, restart level
	call	initLevelMusic

	; R=Restart level
80:	ld	a,(kbdRow4Status)
	bit	7,a
	ret	nz
	ld	b,a
	cpl
	and	#60
	jr	z,.restartLevel
	ld	hl,gameFlags
	set	gameFlags.skipd,(hl)
	bit	6,b
	jr	z,87F
	ld	a,(currentLevel)
	sub	2
	jr	nc,85F
	call	clearAllSprites
	call	clearScreen
	xor	a
85:	ld	(currentLevel),a
87:	call	muteMusic
	ld	a,GS_STAGECLEAR : ld	(gameState),a	
	ret

.restartLevel:
	call	muteMusic
	ld	a,GS_DEAD 	: ld	(gameState),a    
	ret

doShowLevel:
	ld	b,8
90:	push	bc
	call	showLevel
	pop	bc
	ld	a,(gameState)
	cp	GS_SHOWLEVEL
	ret	nz
	djnz	90B

	ld	a,(timer)
	and	3
	ret	nz
	jp	playMostSound

doGameOver:
	ld	a,(joyStatus)
	and	#10
	jp	z,popintro
	ld	hl,(timer)
	ld	de,255*50/10	; wait 25.5s
	rst	DCOMPR
	ret	c
	jp	popintro

;==========================================================================
; Includes

	; MSX System constants and variables
	include "asmlib/equ/MSXBIOS.equ.z80"
	include "asmlib/equ/MSXvars.equ.z80"
	include "asmlib/equ/MSXhooks.equ.z80"
	include "asmlib/equ/MSX-DOS.equ.z80"

	; Game routines that must sit on frame-1 
	include "include/engine.asm"
	include "asmlib/base/slots.inc.z80"

	; Data
	include "maps\maps.a80"
	include "gfx\screen.asm"
	include "gfx\menu.asm"
	include "gfx\gfx.asm"

	; Helper Libraries
	include "include\unmegalz_fast_v2.asm"
;	include	"include\unRLEWBtoVRAM.asm"
	include "asmlib/math/binarymath.z80"
	include "asmlib/math/rand.z80"
	include "asmlib/file/easyfilelib.inc.z80"
	include "asmlib/file/easyfilelib_init.inc.z80"

	; Game subroutines
	include "include\level.asm"
	include "include\menu.asm"
	include "include\strings.asm"
	include "include\enemy.asm"
	include "include\hero.asm"
	include "include\bridge.asm"
	include "include\mines.asm"

	; Platform-specific I/O stuff
	include "include\joyKbd.asm"
	include "include\video.asm"
	include "include\sound.asm"

	; Game routines that must sit on frame-2
	EZFILELIB.macro_placeDskErrRomHdlHere

    display /d,"total: ",$-Start

	ds	#C000-$,#FF	; Unused EPROM space must be filled with FFh

;RAM
    org #C000 ;+8912 for 8k msx
beginRAM:
	include "include\RAM.asm"    
	include "sound\pt3Ram.asm"
	include "sound\ayfx-RAM.asm"

 ALIGN 32
map:	block 768,0
	block 32,0		; Empty row to make sure that hero and enemies
.end:				 ; can fall to the top

musicBuffer: block MUSBUFFERSIZE,0

    display /d,"RAM: ",$
endRAM:

;-----------------------------------------------
; Sanity checks

 IF endRAM > #EE00
	DISPLAY "Warning: RAM usage is worringly high."
 ENDIF


;-----------------------------------------------
 IF (_ERRORS = 0) 
        
            savebin "YAZZIER.ROM",16384,32768 ;32kb ROM
;			SHELLEXEC "Yazzie.rom"

 ENDIF
